# NEST Firmware - Arduino Version (Prototipo Wearable)

Este repositorio contiene el firmware para el proyecto **NEST (Non-intrusive dEviceS for Telemedicine)**, desarrollado para un **prototipo wearable** en la plataforma **Arduino**. El firmware controla el monitoreo de **frecuencia cardíaca (BPM)** mediante el sensor **MAX30102** y el microcontrolador **nRF52840** con **Bluetooth Low Energy (BLE)** integrado. También incluye una interfaz táctil para interactuar con las funcionalidades del dispositivo, como la confirmación de rescates, monitoreo, y envío de datos a través de BLE.

## Estructura del Repositorio

- **/bootloader**: Archivos del bootloader para el nRF52840, que se deben cargar manualmente.
- **/firmware**: Código fuente del firmware desarrollado en Arduino.
- **/libraries**: Bibliotecas necesarias como **Ucglib.h**, **algorithm_by_RF.h**, y **max30102.h** para controlar la pantalla TFT y el sensor de frecuencia cardíaca.

## Descripción del Firmware

Este firmware está diseñado para un wearable médico que permite el monitoreo de frecuencia cardíaca y la interacción con el usuario a través de una pantalla táctil. Las principales funcionalidades del firmware incluyen:

1. **Monitoreo de BPM**: Utiliza el sensor **MAX30102** para medir la frecuencia cardíaca (BPM). Los datos se procesan y se muestran en la pantalla TFT.
2. **Pantalla TFT y Control Táctil**: La interfaz de usuario incluye un menú principal con opciones para monitorear los signos vitales, interactuar con un diario de dolor, y confirmar el envío de datos de rescate. Todo esto se controla mediante una pantalla TFT manejada con la biblioteca **Ucglib.h** y un sistema táctil basado en la biblioteca **TouchScreen.h**.
3. **Conectividad Bluetooth**: Implementa BLE usando **Bluefruit.h** para enviar notificaciones y datos de monitoreo a dispositivos conectados. Incluye características BLE personalizadas para enviar alertas.
4. **Iluminación de Pantalla Controlada**: El pin de iluminación de la pantalla está controlado con PWM, lo que permite ajustar el brillo de la pantalla en función del estado del dispositivo.
5. **Actualización Dinámica de Frecuencia Cardíaca**: El valor de BPM se actualiza en tiempo real y se muestra en la pantalla, y también puede ser enviado vía BLE si hay un dispositivo conectado.

## Funciones Principales

- **Pantalla Principal**: Presenta tres opciones: **Diario de Dolor**, **Rescate**, y **Monitoreo**.
- **Confirmación de Rescate**: Permite al usuario confirmar o cancelar el envío de datos de rescate.
- **Envío de Datos**: Muestra una animación de envío y notifica cuando los datos se han enviado correctamente a través de BLE.
- **Monitoreo de Frecuencia Cardíaca**: Muestra el valor actual de BPM en la pantalla y permite volver al menú principal tocando la pantalla.

### Bibliotecas Incluidas en el Proyecto

El proyecto incluye las siguientes bibliotecas personalizadas que se encuentran en el directorio `/libraries`:

- **Ucglib.h**: Utilizada para controlar la pantalla TFT e implementar las interfaces gráficas.
- **algorithm_by_RF.h**: Procesa los datos del sensor **MAX30102** para calcular la frecuencia cardíaca.
- **max30102.h**: Controla las operaciones del sensor **MAX30102** para la lectura de datos biométricos.

Otras bibliotecas necesarias, como **Adafruit_TinyUSB.h**, **TouchScreen.h**, **Bluefruit.h** y **SPI.h**, se pueden instalar directamente desde el gestor de bibliotecas del **Arduino IDE**.

## Requisitos para Programar en Arduino

### Instalar Arduino Core para nRF52840

1. **Descargar e instalar Arduino IDE** (al menos v1.6.12): [Arduino IDE](https://www.arduino.cc/en/software).
2. Iniciar el Arduino IDE y entrar a las **Preferencias**.
3. En **Additional Board Manager URLs**, agregar:  
   `https://adafruit.github.io/arduino-board-index/package_adafruit_index.json`
4. Reiniciar el Arduino IDE.
5. Ir a **Herramientas > Placa > Gestor de Placas** y buscar "Adafruit nRF52" e instalar **Adafruit nRF52 by Adafruit**.
6. Una vez instalado, seleccionar **Adafruit Feather nRF52840 Express** desde **Herramientas > Placa** para configurar el sistema.

## Bootloader

El bootloader debe cargarse manualmente. En este repositorio, en el directorio `/bootloader`, encontrarás el archivo que debe cargarse en el nRF52840. Sigue las instrucciones a continuación para realizar la grabación del bootloader.

### Grabar un nuevo Bootloader

Para grabar el bootloader desde el IDE de Arduino, necesitarás tener instaladas las siguientes herramientas en tu sistema y que estén disponibles en la ruta del sistema:

- [Segger JLink Software and Documentation Pack](https://www.segger.com/downloads/jlink)
- [Nordic nRF5x Command Line Tools](https://www.nordicsemi.com/Software-and-Tools/Development-Tools/nRF-Command-Line-Tools)

Verifica que puedas ejecutar `nrfjprog` desde tu terminal o símbolo del sistema.

### Grabar el Bootloader manualmente con nrfjprog

El archivo hex del bootloader se puede encontrar en la carpeta `/bootloader`. Ejecuta los siguientes comandos desde la carpeta donde se encuentra el bootloader:

```bash
$ nrfjprog -e -f nrf52
$ nrfjprog --program feather_nrf52840_bootloader.hex -f nrf52
$ nrfjprog --reset -f nrf52
```

### Configuraciones en Arduino

Una vez que el bootloader esté grabado, realiza las siguientes configuraciones en el IDE de Arduino:

1. Selecciona **Herramientas > Placa > Adafruit Feather nRF52840 Express**.
2. Selecciona el puerto COM que corresponda a la placa.
3. En **Herramientas > Programador**, selecciona **J-Link for Bluefruit nRF52**.
4. Luego, selecciona **Herramientas > Grabar Bootloader** para finalizar el proceso de configuración.


